#!/usr/bin/perl
#####################################################################
# Function: common.pm
#
#
# 概要:
# PG-REX 便利ツールから呼び出すモジュールの集まり
# 
# 特記事項:
# なし
#
#####################################################################
use warnings;
use strict;
use Switch;
use Net::OpenSSH;

BEGIN {
#    if ($ENV{"LANG"} =~ m/ja/i){
    if (1){
        eval qq{
            use po::ja;
        };
    }
    else{
        eval qq{
            use po::en;
        }
    }
};


sub postgres_running{
    my ($pgdata) = @_;
    my $result;

    $result = exec_command("ps aux | grep \"bin/postgres -D $pgdata \" | grep -v \"grep\"");

    # PostgreSQL のプロセスを発見したら1を返却する
    if ($result ne ""){
        return 1;
    }
    return 0;
}


sub pacemaker_running{
    my $result;

    $result = exec_command("ps aux | grep \"heartbeat: master control process\" | grep -v \"grep\"");

    # Pacemaker のプロセスを発見したら1を返却する
    if ($result ne ""){
        return 1;
    }
    return 0;
}


sub pacemaker_online{
    my ($my_node, $ssh_pass, $address_connect) = @_;
    my @results;
    my @crm_results;
    my $command;
    my $exit_code;

    $command = "crm_mon -fA -1";

    if ($address_connect && $ssh_pass){
        @results = ssh_exec_command($address_connect, "root", $ssh_pass, $command);
    }
    else {
        $results[0] = `$command`;
        $results[1] = $? >> 8;
    }
    $exit_code = $results[1];
    # Pacemaker が起動している場合は $exit_code = 0 、起動していない場合は $exit_code = 10
    if ($exit_code != 0 && $exit_code != 10){
        printlog("ERROR", COMMON_MS0001);
    }

    # crm_mon の結果を行ごとの配列に格納する
    @crm_results = split (/\n/, $results[0]);

    # Pacemaker が Online だったら1を返却する
    foreach my $line (@crm_results){
        # crm_mon の結果に「Online: [ ～ <マシンのホスト名>」の行が存在すると OK
        if ($line =~ /Online\:\s+\[(\s+\S+)?\s+$my_node\s+/){
            return 1;
        }
    }
    return 0;

}


sub pgrex_failed_action{
    my ($my_node, $ssh_pass, $address_connect) = @_;
    my @results;
    my @crm_results;
    my $command;
    my $exit_code;

    $command = "crm_mon -fA -1";
    if ($address_connect && $ssh_pass){
        @results = ssh_exec_command($address_connect, "root", $ssh_pass, $command);
    }
    else {
        $results[0] = `$command`;
        $results[1] = $? >> 8;
    }
    $exit_code = $results[1];
    # Pacemaker が起動している場合は $exit_code = 0 、起動していない場合は $exit_code = 10
    if ($exit_code != 0 && $exit_code != 10){
        printlog("ERROR", COMMON_MS0001);
    }
    
    # crm_mon の結果を行ごとの配列に格納する
    @crm_results = split (/\n/, $results[0]);

    my $line_count = 0;
    my $line_tmp = 0;
    # crm_mon の結果に failed Action の内容が存在していたら1を返却する
    foreach my $line (@crm_results){
        $line_count++;

        # crm_mon の結果に「Failed actions:」の行が存在した後、
        # 1行後に「pgsql:(.) ～ node=<マシンのホスト名>」の行が存在すると OK
        # ※(.) = 文字の種類は問わず一文字
        if ($line =~ /Failed\s+actions:/){
            $line_tmp = $line_count;
        }
        if ($line =~ /pgsql:[0-1]+_\S+_[0-9]+\s+\(node=$my_node/ && $line_count == $line_tmp + 1){
            return 1;
        }
    }
    return 0;

}


sub network_running{
    my ($my_node, $another_node, @nic_list) = @_;
    my $result;
    my $attribute;
    my $command;
    my $exit_code;

    # コマンド
    # 「crm_attribute -t status -N <ノード名> --name <ノード名>-<NIC名> -G -q」
    # の結果が2ノード×2nicの計4つとも"up"であると OK
    foreach my $nic (@nic_list){
        $attribute = $another_node."-".$nic;
        $command = "crm_attribute -t status -N $my_node --name $attribute -G -q 2> /dev/null";
        $result = `$command`;
        chomp $result;
        if ($result ne "up"){
            return 0;
        }
    }
    foreach my $nic (@nic_list){
        $attribute = $my_node."-".$nic;
        $command = "crm_attribute -t status -N $another_node --name $attribute -G -q 2> /dev/null";
        $result = `$command`;
        chomp $result;
        if ($result ne "up"){
            return 0;
        }
    }
    return 1;

}


sub resource_running{
    my ($my_node, $operation, $ssh_pass, $address_connect) = @_;
    my $result;
    my @results;
    my $command;

    # $operation によって確認事項が異なり、それぞれで戻り値を決定する
    switch ($operation){

        ##
        # "vip-master" を指定した場合、以下の条件を満たしたとき結果を OK とする
        #   *  コマンド
        #     「crm_resource -r vip-master -W 2> /dev/null」
        #      の結果が"resource vip-master is running on: <マシンのホスト名> "である
        ##
        case "vip-master" {
            $result = `crm_resource -r vip-master -W 2> /dev/null`;
            chomp $result;
            if ($result ne "resource vip-master is running on: $my_node "){
                return 0;
            }
            return 1;
        }

        ##
        # "vip-slave" を指定した場合、以下の条件を満たしたとき結果を OK とする
        #   *  コマンド
        #     「crm_resource -r vip-slave -W 2> /dev/null」
        #      の結果が"resource vip-slave is running on: <マシンのホスト名> "である
        ##
        case "vip-slave" {
            $result = `crm_resource -r vip-slave -W 2> /dev/null`;
            chomp $result;
            if ($result ne "resource vip-slave is running on: $my_node "){
                return 0;
            }
            return 1;
        }

        ##
        # "vip-rep" を指定した場合、以下の条件を満たしたとき結果を OK とする
        #   *  コマンド
        #     「crm_resource -r vip-rep -W 2> /dev/null」
        #      の結果が"resource vip-rep is running on: <マシンのホスト名> "である
        ##
        case "vip-rep" {
            $result = `crm_resource -r vip-rep -W 2> /dev/null`;
            chomp $result;
            if ($result ne "resource vip-rep is running on: $my_node "){
                return 0;
            }
            
            return 1;
        }

        ##
        # "stonith" を指定した場合、以下の条件を満たしたとき結果を OK とする
        #   *  コマンド
        #     「crm_resource -r grpStonith1 -W 2> /dev/null」
        #     「crm_resource -r grpStonith2 -W 2> /dev/null」
        #      のどちらも実行して、どちらかの結果が
        #      "resource grpStonithX is running on: <マシンのホスト名> "である
        ##
        case "stonith" {
            $result = `crm_resource -r grpStonith1 -W 2> /dev/null`;
            chomp $result;
            if ($result eq "resource grpStonith1 is running on: $my_node "){
                return 1;
            }
            $result = `crm_resource -r grpStonith2 -W 2> /dev/null`;
            chomp $result;
            if ($result eq "resource grpStonith2 is running on: $my_node "){
                return 1;
            }
            return 0;
        }

        ##
        # "pg-rex_master" を指定した場合、以下の条件を満たしたとき結果を OK とする
        #   *  コマンド
        #     「crm_resource -r msPostgresql -W 2> /dev/null | grep " <マシンのホスト名> "」
        #      の結果が"resource msPostgresql is running on: <マシンのホスト名> Master"である
        #   *  コマンド
        #     「crm_attribute -t status -N <マシンのホスト名> --name master-pgsql:0 -G -q」
        #     「crm_attribute -t status -N <マシンのホスト名> --name master-pgsql:1 -G -q」
        #      のどちらも実行して、どちらか結果が"1000"である
        #   *  コマンド
        #     「crm_attribute -l forever -N <マシンのホスト名> --name pgsql-data-status -G -q」
        #      の結果が"LATEST"である
        #   *  コマンド
        #     「crm_attribute -t status -N <マシンのホスト名> --name pgsql-status -G -q」
        #      の結果が"PRI"である
        ##
        case "pg-rex_master" {
            $command = "crm_resource -r msPostgresql -W 2> /dev/null | grep \" $my_node \"";
            if ($address_connect && $ssh_pass){
                @results = ssh_exec_command($address_connect, "root", $ssh_pass, $command);
            }
            else {
                $results[0] = `$command`;
                chomp $results[0];
            }
            
            if ($results[0] ne "resource msPostgresql is running on: $my_node Master"){
                return 0;
            }

            $command = "crm_attribute -l forever -N $my_node --name pgsql-data-status -G -q 2> /dev/null";
            if ($address_connect && $ssh_pass){
                @results = ssh_exec_command($address_connect, "root", $ssh_pass, $command);
            }
            else {
                $results[0] = `$command`;
                chomp $results[0];
            }
            if ($results[0] ne "LATEST"){
                return 0;
            }

            $command = "crm_attribute -t status -N $my_node --name pgsql-status -G -q 2> /dev/null";
            if ($address_connect && $ssh_pass){
                @results = ssh_exec_command($address_connect, "root", $ssh_pass, $command);
            }
            else {
                $results[0] = `$command`;
                chomp $results[0];
            }
            if ($results[0] ne "PRI"){
                return 0;
            }

            $command = "crm_attribute -t status -N $my_node --name master-pgsql:0 -G -q 2> /dev/null";
            if ($address_connect && $ssh_pass){
                @results = ssh_exec_command($address_connect, "root", $ssh_pass, $command);
            }
            else {
                $results[0] = `$command`;
                chomp $results[0];
            }
            if ($results[0] eq "1000"){
                return 1;
            }

            $command = "crm_attribute -t status -N $my_node --name master-pgsql:1 -G -q 2> /dev/null";
            if ($address_connect && $ssh_pass){
                @results = ssh_exec_command($address_connect, "root", $ssh_pass, $command);
            }
            else {
                $results[0] = `$command`;
                chomp $results[0];
            }
            if ($results[0] eq "1000"){
                return 1;
            }

            return 0;
        }

        ##
        # "pg-rex_slave" を指定した場合、以下の条件を満たしたとき結果を OK とする
        #   *  コマンド
        #     「crm_resource -r msPostgresql -W 2> /dev/null | grep " <マシンのホスト名> "」
        #      の結果が"resource msPostgresql is running on: <マシンのホスト名> "である
        #   *  コマンド
        #     「crm_attribute -t status -N <マシンのホスト名> --name master-pgsql:1 -G -q」
        #     「crm_attribute -t status -N <マシンのホスト名> --name master-pgsql:0 -G -q」
        #      のどちらも実行して、どちらかの結果が"100"である
        #   *  コマンド
        #     「crm_attribute -l forever -N <マシンのホスト名> --name pgsql-data-status -G -q」
        #      の結果が"STREAMING|SYNC"である
        #   *  コマンド
        #     「crm_attribute -t status -N <マシンのホスト名> --name pgsql-status -G -q」
        #      の結果が"HS:SYNC"である
        ##
        case "pg-rex_slave" {
            $command = "crm_resource -r msPostgresql -W 2> /dev/null | grep \" $my_node \"";
            if ($address_connect && $ssh_pass){
                @results = ssh_exec_command($address_connect, "root", $ssh_pass, $command);
            }
            else {
                $results[0] = `$command`;
                chomp $results[0];
            }
            if ($results[0] ne "resource msPostgresql is running on: $my_node "){
                return 0;
            }

            $command = "crm_attribute -l forever -N $my_node --name pgsql-data-status -G -q 2> /dev/null";
            if ($address_connect && $ssh_pass){
                @results = ssh_exec_command($address_connect, "root", $ssh_pass, $command);
            }
            else {
                $results[0] = `$command`;
                chomp $results[0];
            }
            if ($results[0] ne "STREAMING|SYNC"){
                return 0;
            }

            $command = "crm_attribute -t status -N $my_node --name pgsql-status -G -q 2> /dev/null";
            if ($address_connect && $ssh_pass){
                @results = ssh_exec_command($address_connect, "root", $ssh_pass, $command);
            }
            else {
                $results[0] = `$command`;
                chomp $results[0];
            }
            if ($results[0] ne "HS:sync"){
                return 0;
            }

            $command = "crm_attribute -t status -N $my_node --name master-pgsql:1 -G -q 2> /dev/null";
            if ($address_connect && $ssh_pass){
                @results = ssh_exec_command($address_connect, "root", $ssh_pass, $command);
            }
            else {
                $results[0] = `$command`;
                chomp $results[0];
            }
            if ($results[0] eq "100"){
                return 1;
            }

            $command = "crm_attribute -t status -N $my_node --name master-pgsql:0 -G -q 2> /dev/null";
            if ($address_connect && $ssh_pass){
                @results = ssh_exec_command($address_connect, "root", $ssh_pass, $command);
            }
            else {
                $results[0] = `$command`;
                chomp $results[0];
            }
            if ($results[0] eq "100"){
                return 1;
            }
            return 0;
        }

        ##
        # "pingd" を指定した場合、以下の条件を満たしたとき結果を OK とする
        #   *  コマンド
        #     「crm_resource -r clnPingd -W 2> /dev/null | grep " <マシンのホスト名> "」
        #      の結果が"resource clnPingd is running on: <マシンのホスト名> "である
        #   *  コマンド
        #     「crm_attribute -t status -N <マシンのホスト名> --name default_ping_set -G -q」
        #      の結果が"100"である
        ##
        case "pingd" {
            $result = `crm_resource -r clnPingd -W 2> /dev/null | grep \" $my_node \"`;
            chomp $result;
            if ($result ne "resource clnPingd is running on: $my_node "){
                return 0;
            }

            $result = `crm_attribute -t status -N $my_node --name default_ping_set -G -q 2> /dev/null`;
            chomp $result;
            if ($result ne "100"){
                return 0;
            }

            return 1;
        }

        ##
        # "diskd" を指定した場合、以下の条件を満たしたとき結果を OK とする
        #   *  コマンド
        #     「crm_resource -r clnDiskd1 -W 2> /dev/null | grep " <マシンのホスト名> "」
        #      の結果が"resource clnDiskd1 is running on: <マシンのホスト名> "である
        #   *  コマンド
        #     「crm_attribute -t status -N <マシンのホスト名> --name diskcheck_status_internal -G -q」
        #      の結果が"normal"である
        ##
        case "diskd" {
            $result = `crm_resource -r clnDiskd1 -W 2> /dev/null | grep \" $my_node \"`;
            chomp $result;
            if ($result ne "resource clnDiskd1 is running on: $my_node "){
                return 0;
            }
        ##
        # diskcheck_status_internalのチェックで便利ツールの起動がタイムアウトしないように、
        # 下記の該当場所をコメントアウトしておく。
        ##
       
        #    $result = `crm_attribute -t status -N $my_node --name diskcheck_status_internal -G -q 2> /dev/null`;
        #    chomp $result;
        #    if ($result ne "normal"){
        #        return 0;
        #    }

            return 1;
        }

        ##
        # 上記以外の値を指定した場合、エラーを出力とする
        ##
        else {
            printlog("ERROR", COMMON_MS0002, $operation);
        }
    }
}


sub read_cib{
    my (@cib_strings) = @_;
    my %cib_value;
    my $my_node;
    my $my_node_id;
    my $my_pgsql_data_status_id;
    my @key_list = ("pgdata", "master_ip", "repuser", "tmpdir");
    my $result;

    $my_node = exec_command("uname -n");
    chomp $my_node;

    # PG-REX のバージョンを確認し、その結果に合わせて tmpdir のデフォルト値を設定する
    $result = get_pg_version();
    if ($result =~ /^9\.1[^0-9]/){
        $cib_value{"tmpdir"} = "/var/lib/pgsql";
    }
    else{
        $cib_value{"tmpdir"} = "/var/lib/pgsql/tmp";
    }

    foreach my $line (@cib_strings){
        # pgsql-instance_attributes format : 
        #     <nvpair id="pgsql-instance_attributes-start_opt" name="<name>" value="<value>"/>
        if ($line =~ /\"pgsql-instance_attributes-\S+\"\s+name=\"([^\"\s]+)\"\s+value=\"([^\"]+)\"/){
            $cib_value{$1} = $2;
        }
        # node id format : 
        #     <node (id="<node id>" type="<type>" uname="<node name>")>
        #     ※()の中のタグの順番は不定
        if ($line =~ /uname=\"$my_node\"/ && $line =~ /id=\"([^\"\s]+)\"/){
            $my_node_id = $1;
            $my_pgsql_data_status_id = "nodes-".$my_node_id."-pgsql-data-status";
        }
    }

    foreach my $line (@cib_strings){
        if ($my_pgsql_data_status_id && $line =~ /id=\"$my_pgsql_data_status_id\"/ && $line =~ /value=\"([^\"\s]+)\"/){
            $cib_value{"pgsql_data_status"} = $1;
        }
    }
    
    foreach my $key (@key_list) {
        if (!exists($cib_value{$key})){
            printlog("ERROR", COMMON_MS0003, $key);
        }
    }

    return %cib_value;
}


sub read_config{
    my (@config_strings) = @_;
    my %config_value;
    my @key_list = ("Archive_dir", "S-LAN_IPAddress", "STONITH","VIP_SLAVE");
    my $result;

    $config_value{'STONITH'} = "enable";
    $config_value{'VIP_SLAVE'} = "enable";

    foreach my $line (@config_strings){
        # コメントの削除
        # comment format : #<comment>
        $line =~ s/\#.*//g;

        # 行末の空白の削除
        # space format : <space><\n>
        $line =~ s/\s+$//g;

        # 全角の空白を半角空白に変換
        # two-byte characterspace format : <two-byte character space>
        $line =~ s/　/ /g;

        # environment setting value format : <<value name> = <parameter>> or <<value name> = <parameter>,<parameter>>
        if ($line =~ /^\s*(\S+)\s*=\s*(.+)$/){
            $config_value{$1} = $2;
        }
    }

    foreach my $key (@key_list) {
        if (!exists($config_value{$key})){
            printlog("ERROR", COMMON_MS0004, $key);
        }
    }
    if ($config_value{'Archive_dir'} !~ /^\//){
        printlog("ERROR", COMMON_MS0005);
    }
    if ($config_value{'STONITH'} ne "enable" && $config_value{'STONITH'} ne "disable"){
        printlog("ERROR", COMMON_MS0006);
    }
    if ($config_value{'VIP_SLAVE'} ne "enable" && $config_value{'VIP_SLAVE'} ne "disable"){
        printlog("ERROR", COMMON_MS0021);
    }

    my @ifconfig_ip = ();
    # ","で区切られた S-LAN の IP アドレスをそれぞれ配列に格納
    my @s_lan_ip = split(/\s*,\s*/, $config_value{"S-LAN_IPAddress"});
    if (scalar(@s_lan_ip) != 2){
        printlog("ERROR", COMMON_MS0007);
    }

    # コマンド実行マシンに設定してある IP アドレスを配列に格納
    $result = exec_command("ifconfig | grep \"inet addr\"");
    my @ifconfig_strings = split(/\n/, $result);
    foreach my $line (@ifconfig_strings){
        # IP address format : <inet addr:<value>>
        if ($line =~ /inet\saddr:([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+)/) {
            push (@ifconfig_ip , $1);
        }
    }
    if (!scalar(@ifconfig_ip)){
        printlog("ERROR", COMMON_MS0008);
    }

    foreach my $line (@s_lan_ip){
        if (grep {$_ eq $line} @ifconfig_ip){
            $config_value{"My_S-LAN_IPAddress"} = $line;
        }
        else {
            $config_value{"Another_S-LAN_IPAddress"} = $line;
        }
    }

    if (!exists($config_value{'My_S-LAN_IPAddress'})){
        printlog("ERROR", COMMON_MS0009);
    }
    if (!exists($config_value{'Another_S-LAN_IPAddress'})){
        printlog("ERROR", COMMON_MS0010);
    }

    return %config_value;
}


sub read_hacf{
    my (@hacf_strings) = @_;
    my %hacf_value;
    my @ic_dev;
    my @node;

    # IC-LAN に使用しているデバイス名とノード名を取得
    foreach my $line (@hacf_strings){
        chomp $line;
        # comment format : #<comment>
        $line =~ s/\#.*//g;
        # IC-LAN device name format : bcast <device name> or ucast <device name>
        if ($line =~ /^\s*(b|u)cast\s+(\S+)/){
            push(@ic_dev, $2);
        }
        # node name format : node <node name>
        if ($line =~ /^\s*node\s+(\S+)/){
            push(@node, $1);
        }
    }
    if (scalar(@ic_dev) != 2){
        printlog("ERROR", COMMON_MS0019);
    }
    if (scalar(@node) != 2){
        printlog("ERROR", COMMON_MS0020);
    }

    %hacf_value = (ic_dev => [$ic_dev[0], $ic_dev[1]],
                   node   => [$node[0], $node[1]]);

    return %hacf_value;
}


sub exec_command {
    my ($command) = @_;

    my $return = `$command`;
    my $exit_code = $? >> 8;
    if ($exit_code != 0 && $exit_code != 1){
        printlog("ERROR", COMMON_MS0011, $command);
    }

    return $return;
}


sub ssh_exec_command {
    my ($address, $user, $pass, $command) = @_;
    my @results;
    
    my $ssh = Net::OpenSSH->new($address, user => $user, password => $pass);
    $ssh->error and printlog("ERROR", COMMON_MS0016);
    
    $results[0] = $ssh->capture($command);
    $results[1] = $? >> 8;
    chomp $results[0];
    
    return @results;
}


sub rsync_exec_command {
    my ($address, $user, $pass, $remote, $local) = @_; 
    
    my $ssh = Net::OpenSSH->new($address, user => $user, password => $pass);
    $ssh->error and printlog("ERROR", COMMON_MS0016);
    
    my %opts = (
        archive => 1,
        verbose => 1,
    );
    
    $ssh-> rsync_get(\%opts, $remote, $local) or $ssh->error and printlog("ERROR", COMMON_MS0017);
    
    return 0;
}


sub scp_exec_command {
    my ($address, $user, $pass, $remote, $local) = @_; 
    
    my $ssh = Net::OpenSSH->new($address, user => $user, password => $pass);
    $ssh->error and printlog("ERROR", COMMON_MS0016);
    
    $ssh-> scp_get({copy_attrs => 1}, $remote, $local) or $ssh->error and printlog("ERROR", COMMON_MS0018);
    
    return 0;
}


sub get_xlogfile_name{
    my ($location, $ThisTimeLineID) = @_;
    my $uxlogid;
    my $uxrecoff;
    my $xlogid;
    my $xlogseg;
    my $xlogfilename;
    
    # XLOG 位置の情報をパース
    # location format : <xlogid>/<xrecoff>
    # <xlogid>は1,2桁の16進数
    # <xrecoff>は7.8桁の16進数
    if ($location !~ /^([0-9A-F]{1,2})\/([0-9A-F]{7,8})$/){
        printlog("ERROR", COMMON_MS0013);
    }
    $uxlogid = $1;
    $uxrecoff = $2;

    # 一つ前のセグメントを取得
    # $xlogsegは、上で取得した16進数の値から1を引いて5桁以下をシフトした値になる
    $xlogid = hex($uxlogid);
    my $uxrecoff_tmp = hex($uxrecoff);
    $xlogseg = sprintf("%d", ($uxrecoff_tmp - 1)/(16 ** 6));

    # 一つ前のセグメントの WAL ファイル名を取得
    $xlogfilename = sprintf("%08X%08X%08X", $ThisTimeLineID, $xlogid, $xlogseg);

    return $xlogfilename;
}


sub get_controldata_value{
    my (@controldata_strings) = @_;
    my %controldata_value;
    my @key_list = ("updatetime", "database", "timelineid", "xloglocation");

    foreach my $line (@controldata_strings){
        # pg_control last modified format : pg_control last modified: <Wday> <Mon> <Mday> <Hour>:<Min>:<Sec> <Year>
        if ($line =~ /^pg_control\s+last\s+modified:\s+(.+)$/){
            $controldata_value{'updatetime'} = $1;
        }
        # Database system identifier format : Database system identifier: <Database>
        if ($line =~ /^Database\s+system\s+identifier:\s+([0-9A-F]+)$/){
            $controldata_value{'database'} = $1;
        }
        # TimeLineID format : Latest checkpoint's TimeLineID: <TimeLineID>
        # <TimeLineID>は10進数
        if ($line =~ /^Latest\s+checkpoint's\s+TimeLineID:\s+([0-9A-F]+)$/){
            $controldata_value{'timelineid'} = $1;
        }
        # location format : Latest checkpoint's REDO location: <location>
        if ($line =~ /^Latest\s+checkpoint's\s+REDO\s+location:\s+([0-9A-F\/]+)$/){
            $controldata_value{'xloglocation'} = $1;
        }
    }

    foreach my $key (@key_list) {
        if (!exists($controldata_value{$key})){
            printlog("ERROR", COMMON_MS0014);
        }
    }


    return %controldata_value;
}


sub get_recoverywal {
    my (@controldata_strings) = @_;
    my $wal = '';

    my %controldata_value = get_controldata_value(@controldata_strings);

    $wal = get_xlogfile_name($controldata_value{'xloglocation'}, $controldata_value{'timelineid'});

    return $wal;
}


sub get_node{
    my ($another_address,$ssh_pass) = @_;
    my @results;
    my %node_value = ("my_node"      => '',
                      "another_node" => '');

    $node_value{'my_node'} = exec_command("uname -n");
    chomp $node_value{'my_node'};

    @results = ssh_exec_command($another_address, "root", $ssh_pass, "uname -n");
    $node_value{'another_node'} = $results[0];
    chomp $node_value{'another_node'};

    if (!$node_value{'another_node'}){
        printlog("ERROR", COMMON_MS0015);
    }

    return %node_value;
}


sub get_pg_version{
    my $result;
    my @results;

    $result = exec_command("su - postgres -c \"pg_basebackup --version \"");

    @results = split(/\s/,$result);
    chomp $results[2];

    return $results[2];
}


sub printlog {
    my ($log_level, $message, @values) = @_;

    for (my $count=0; $count < scalar(@values); $count++){
        $message =~ s/\[$count\]/$values[$count]/;
    }

    if ($log_level eq "ERROR"){
        die $message;
    }
    else{
        print $message;
    }

}


1;
